/**
 * @file   control.h
 * @author Jan Moravec (jan.moravec@workswell.cz)
 * @copyright (c) 2017, Workswell s.r.o., All rights reserved.
 * @date   March, 2017
 * @brief  This files contains functions for connecting and using the control server on WIRIS device.
 */

#ifndef SRC_LIB_CONTROL_H_
#define SRC_LIB_CONTROL_H_

#include <stdint.h>
#include <string>

/// Connect to server - IP address and port number
int connectControl(char *server, int port);
/// Close connection
int closeControl(void);

/// Basic functions receiving data
int receiveControl(uint8_t *buf, int length, int tmout);
/// Basic functions for sending
int sendControl(const char *mess, int length);

/// Check connection, -1 if error
int checkControl(void);
/// Do image calibration
int doCIC(void);
/// Set temperature range to high
int setRangeHigh(void);
/// Set temperature range to low
int setRangeLow(void);
/// Return temperature range in degrees Celsius
int getRange(int &lowval, int &highval);
/// Return WIRIS device serial number
int getSerialNumber(std::string &serial);
/// Return WIRIS firmware version
int getFirmwareVersion(std::string &version);
/// Return WIRIS DIO version
int getDIOVersion(std::string &version);
/// Get Thermo camera resolution
int getResolutionThermo(int &width, int &height);
/// Get visible camera resolution, depends on scale
int getResolutionVisible(int &width, int &height);
/// Set scale 0=1600x1200, 1=800x600, 2=400x300
int setVisibleScale(int scale);
/// Returns WIRIS device temperature in degrees Celsius
int getDeviceTemperature(double &temperature);
/// Returns WIRIS thermal camera sensor temperature in degrees Celsius
int getSensorTemperature(double &temperature);
/// Set time to correction in minutes [2, 30]
int setTimeToCorrection(int timeMin);
/// Set emissivity [0.5, 1.0]
int setEmissivity(double emis);
/// Get current emissivity
int getEmissivity(double &emis);
/// Set atmospheric temperature [-50.0, 300.0]
int setAtmosphericTemp(double temp);
/// Get current atmospheric temperature
int getAtmosphericTemp(double &temp);
/// Set reflected temperature [-50.0, 300.0]
int setReflectedTemp(double temp);
/// Get reflected temperature
int getReflectedTemp(double &temp);
/// Get current WIRIS date and time
int getDateTime(int &year, int &month, int &day, int &hour, int &minute, int &second);
/// Set WIRIS date and time
int setDateTime(int year, int month, int day, int hour, int minute, int second);
/// Get free memory in per cent
int getFreeMemory(double &spaceRel);
/// Get number of images in internal memory
int getNumberImages(int &images);
/// Get sequence length in seconds in internal memory
int getSequenceLength(int &seqS);
/// Capture images to internal memory according to setting
int captureImages(void);
/// Start capturing sequence
int startSequence(void);
/// Finish capturing sequence
int stopSequence(void);
/// Get sequence capturing status, true if in progress
int getSequence(bool &capturing);
/// Set if the radiometric jpeg should be captured
int setCaptureRadiometric(bool capture);
/// Set if the non-radiometric jpeg should be captured
int setCaptureScreen(bool capture);
/// Set if the TIFF should be captured
int setCaptureTiff(bool capture);
/// Set if the visible jpeg should be captured
int setCaptureVisible(bool capture);
/// Set palette for thermal image capture
int setPalette(std::string palette);
/// Set range manual or automatic
int setRangeManual(bool manual);
/// Set minimum for manual range
int setRangeMinimum(double minimum);
/// Set minimum for maximum range
int setRangeMaximum(double maximum);
/// Get GPS status and coordiantes
int getGPS(bool &status, double &latitude, double &longitude, double &altitude);
/// Stop ethernet mode, start HDMI stream
int stopEthernetMode();
/// Shut down WIRIS device
int shutDownWiris();
/// Reboot WIRIS device
int rebootWiris();

#endif /* SRC_LIB_CONTROL_H_ */
