/**
 * @file   thermo.cpp
 * @author Jan Moravec (jan.moravec@workswell.cz)
 * @copyright (c) 2017, Workswell s.r.o., All rights reserved.
 * @date   March, 2017
 */

#include "thermo.h"

#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <unistd.h>
#include <string>
#include <cstring>
#include <arpa/inet.h>
#include <sys/socket.h>

static int socketDesc = -1;
static struct sockaddr_in socketAddr;
static int socketAddrLen = sizeof(socketAddr);

int connectThermo(char *server, int port)
{
    if ( (socketDesc = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1 )
    {
    	std::cerr << "connectThermo(): Socket error" << std::endl;
    	return -1;
    }

    memset((char *) &socketAddr, 0, sizeof(socketAddr));
    socketAddr.sin_family = AF_INET;
    socketAddr.sin_port = htons(port);

    if (inet_aton(server , &socketAddr.sin_addr) == 0)
    {
        std::cerr << "connectThermo(): inet_aton error" << std::endl;
        return -1;
    }

    return 0;
}

int closeThermo(void)
{
	if(socketDesc == -1) {
		return 0;
	}

	close(socketDesc);
	socketDesc = -1;
	return 0;
}

int receiveThermo(uint8_t *buf, int length, int tmout)
{
	static struct timeval tv; // Timeout
	tv.tv_sec = tmout/1000000;
	tv.tv_usec = tmout%1000000;
	setsockopt(socketDesc, SOL_SOCKET, SO_RCVTIMEO, (char *)&tv,sizeof(struct timeval));

	int bytes_num;
    if ( (bytes_num = recvfrom(socketDesc, buf, length, 0, (struct sockaddr *) &socketAddr, (socklen_t *) &socketAddrLen)) == -1 )
    {
    	return -1;
    }

	return bytes_num;
}

int sendThermo(const char *mess, int length)
{
	int bytes_num;
	if ( (bytes_num = sendto(socketDesc, mess, length , 0, (struct sockaddr *) &socketAddr, socketAddrLen)) == -1)
	{
		return -1;
	}

	return bytes_num;
}

int checkThermo(void)
{
	std::string mess = "HIWS";
	uint8_t buff[16];

	int return_value = sendThermo(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveThermo(buff, 16, 200000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::cout << "checkThermo(): WIRIS answer: " << ans << std::endl;

	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
}

int getFrameThermo(uint16_t *buf, int width, int height)
{
	std::string mess = "GTFM";
	uint8_t *buff_8 = (uint8_t *) buf;

	int return_value = sendThermo(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	int recieved = 0;
	int counter = 0;
	while (recieved < width*height*2)
	{
		if( (return_value = receiveThermo(&buff_8[recieved], 1024*32, 200000)) < 0)
		{
			std::cerr << "getFrameThermo(): Error receiving, received just " << recieved << " bytes out of" << width*height*2 << " bytes" << std::endl;
			break;
		}
		recieved += return_value;
		counter++;
	}

	return recieved;
}

int getFrameThermoNumbered(uint16_t *buf, int width, int height)
{
	std::string mess = "GTFN";
	uint8_t *buff_8 = (uint8_t *) buf;

	int return_value = sendThermo(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	int recieved = 0;
	int counter = 0;
	const int part_size = 1024;
	uint8_t buff_p[1024 + 4];
	while (counter < width*height*2 / part_size)
	{
		if( (return_value = receiveThermo(buff_p, part_size + 4, 200000)) < 0)
		{
			std::cerr << "getFrameThermo(): Error receiving, received just " << recieved << " bytes out of" << (part_size + 4)*(width*height*2 / part_size) << " bytes" << std::endl;
			break;
		}
		recieved += return_value;
		counter++;
		if(buff_p[0] != 0) {
			continue; // Broken packet
		}
		int part = (buff_p[3] << 8) | buff_p[2]; // Packet part number
		if(part > (width*height*2 / part_size - 1) || part < 0) {
			continue; // Broken packet
		}
		memcpy(&buff_8[part*part_size], &buff_p[4], part_size);
	}

	return recieved;
}

double rawToTemperatureK(uint16_t pixRaw)
{
	return (pixRaw / 25.0);
}

double rawToTemperatureC(uint16_t pixRaw)
{
	return (pixRaw / 25.0 - 273.15);
}
