/**
 * @file   visible.cpp
 * @author Jan Moravec (jan.moravec@workswell.cz)
 * @copyright (c) 2017, Workswell s.r.o., All rights reserved.
 * @date   March, 2017
 * @brief  This files contains functions for connecting and streaming the visible images from WIRIS visible port.
 */

#include "visible.h"

#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <unistd.h>
#include <string>
#include <cstring>
#include <arpa/inet.h>
#include <sys/socket.h>

static int socketDesc = -1;
static struct sockaddr_in socketAddr;
static int socketAddrLen = sizeof(socketAddr);

int connectVisible(char *server, int port)
{
    if ( (socketDesc = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1 )
    {
    	std::cerr << "connectVisible(): Socket error" << std::endl;
    	return -1;
    }

    memset((char *) &socketAddr, 0, sizeof(socketAddr));
    socketAddr.sin_family = AF_INET;
    socketAddr.sin_port = htons(port);

    if (inet_aton(server , &socketAddr.sin_addr) == 0)
    {
        std::cerr << "connectVisible(): inet_aton error" << std::endl;
        return -1;
    }

    return 0;
}

int closeVisible(void)
{
	if(socketDesc == -1) {
		return 0;
	}

	close(socketDesc);
	socketDesc = -1;
	return 0;
}

int receiveVisible(uint8_t *buf, int length, int tmout)
{
	static struct timeval tv; // Timeout
	tv.tv_sec = tmout/1000000;
	tv.tv_usec = tmout%1000000;
	setsockopt(socketDesc, SOL_SOCKET, SO_RCVTIMEO, (char *)&tv,sizeof(struct timeval));

	int bytes_num;
    if ( (bytes_num = recvfrom(socketDesc, buf, length, 0, (struct sockaddr *) &socketAddr, (socklen_t *) &socketAddrLen)) == -1 )
    {
    	return -1;
    }

	return bytes_num;
}

int sendVisible(const char *mess, int length)
{
	int bytes_num;
	if ( (bytes_num = sendto(socketDesc, mess, length , 0, (struct sockaddr *) &socketAddr, socketAddrLen)) == -1)
	{
		return -1;
	}

	return bytes_num;
}

int checkVisible(void)
{
	std::string mess = "HIWS";
	uint8_t buff[16];

	int return_value = sendVisible(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveVisible(buff, 16, 200000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::cout << "checkVisible(): WIRIS answer: " << ans << std::endl;

	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
}

int getFrameVisible(uint8_t *buf, int width, int height)
{
	std::string mess = "GVFM";

	int return_value = sendVisible(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	int recieved = 0;
	int counter = 0;
	while (recieved < width*height*2)
	{
		if( (return_value = receiveVisible(&buf[recieved], 1024*32, 150000)) < 0)
		{
			std::cerr << "getFrameVisible(): Error receiving, received just " << recieved << " bytes out of" << width*height*2 << " bytes" << std::endl;
			break;
		}
		recieved += return_value;
		counter++;
	}

	return recieved;
}

int getFrameVisibleNumbered(uint8_t *buf, int width, int height)
{
	std::string mess = "GVFN";

	int return_value = sendVisible(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	int recieved = 0;
	int counter = 0;
	const int part_size = 3200;
	uint8_t buff_p[part_size + 4];
	while (counter < width*height*2 / part_size)
	{
		if( (return_value = receiveVisible(buff_p, part_size + 4, 200000)) < 0)
		{
			std::cerr << "getFrameVisible(): Error receiving, received just " << recieved << " bytes out of" << (part_size + 4)*(width*height*2 / part_size) << " bytes" << std::endl;
			break;
		}
		recieved += return_value;
		counter++;
		if(buff_p[0] != 0) {
			continue; // Broken packet
		}
		int part = (buff_p[3] << 8) | buff_p[2]; // The packet part number
		if(part > (width*height*2 / part_size - 1) || part < 0) {
			continue; // Broken packet
		}
		memcpy(&buf[part*part_size], &buff_p[4], part_size);
	}

	return recieved;
}
