/**
 * @file   main.cpp
 * @author Jan Moravec (jan.moravec@workswell.cz)
 * @copyright (c) 2017, Workswell s.r.o., All rights reserved.
 * @date   March, 2017
 * @brief  This files contains some basic examples how to connect and use the WIRIS Ethernet plugin.
 */


#include <iostream>
#include <unistd.h>
#include <ctime>

#include "lib/thermo.h"
#include "lib/visible.h"
#include "lib/control.h"
#include "lib/processImage.h"
#include "lib/saveImage.h"


int main(int argc , char *argv[])
{
	std::cout << "Starting tests." << std::endl;
	char *ip_address = "10.0.0.209";
	int port_control = 2240;
	int port_thermo = 2241;
	int port_visible = 2242;
	int return_value;

	std::cout << "Connecting to IP and ports of WIRIS." << std::endl;
	return_value = connectControl(ip_address, port_control);
	if(return_value != 0) {
		return -1;
	}
	return_value = connectThermo(ip_address, port_thermo);
	if(return_value != 0) {
		return -1;
	}
	return_value = connectVisible(ip_address, port_visible);
	if(return_value != 0) {
		return -1;
	}

	std::cout << "Waiting for camera to get ready." << std::endl;
	sleep(2);

	/**************************************************************************************************/

	std::cout << "Checking connection." << std::endl;
	return_value = checkControl();
	if(return_value != 0) {
		return -1;
	}
	return_value = checkThermo();
	if(return_value != 0) {
		return -1;
	}
	return_value = checkVisible();
	if(return_value != 0) {
		return -1;
	}

	std::cout << "Running control tests:" << std::endl;
	std::cout << "	- Doing camera image corrections" << std::endl;
	return_value = doCIC();
	if(return_value != 0) {
		return -1;
	}

	int rangeLow, rangeHigh;
	return_value = getRange(rangeLow, rangeHigh);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Thermo range: <" << rangeLow << ", " << rangeHigh << ">" << std::endl;

	std::cout << "	- Setting thermo range to low" << std::endl;
	return_value = setRangeLow();
	if(return_value != 0) {
		return -1;
	}

	return_value = getRange(rangeLow, rangeHigh);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Thermo range: <" << rangeLow << ", " << rangeHigh << ">" << std::endl;

	std::string serial;
	return_value = getSerialNumber(serial);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- WIRIS serial number: " << serial << std::endl;

	std::string version;
	return_value = getFirmwareVersion(version);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- WIRIS firmware version: " << version << std::endl;

	std::string versionDIO;
	return_value = getDIOVersion(versionDIO);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- WIRIS DIO version: " << versionDIO << std::endl;

	int widthThermo, heightThermo;
	return_value = getResolutionThermo(widthThermo, heightThermo);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Thermo resolution: " << widthThermo << "x" << heightThermo << std::endl;

	return_value = setVisibleScale(2);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Visible scale: " << 2 << std::endl;

	int widthVisible, heightVisible;
	return_value = getResolutionVisible(widthVisible, heightVisible);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Visible resolution: " << widthVisible << "x" << heightVisible << std::endl;

	double temperatureDevice;
	return_value =  getDeviceTemperature(temperatureDevice);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Device temperature: " << temperatureDevice << std::endl;

	double temperatureSensor;
	return_value = getSensorTemperature(temperatureSensor);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Sensor temperature: " << temperatureSensor << std::endl;

	std::cout << "	- Setting emissivity" << std::endl;
	return_value = setEmissivity(0.95);
	if(return_value != 0) {
		return -1;
	}

	return_value = setTimeToCorrection(5);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Time to correction: " << 5 << " min" << std::endl;

	double emissivity;
	return_value = getEmissivity(emissivity);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Emissivity: " << emissivity << std::endl;

	std::cout << "	- Setting atmospheric temperature" << std::endl;
	return_value = setAtmosphericTemp(20.0);
	if(return_value != 0) {
		return -1;
	}

	double temperatureAtmospheric;
	return_value = getAtmosphericTemp(temperatureAtmospheric);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Atmospheric temperature: " << temperatureAtmospheric << std::endl;

	std::cout << "	- Setting reflected temperature" << std::endl;
	return_value = setReflectedTemp(20.0);
	if(return_value != 0) {
		return -1;
	}

	double temperatureReflected;
	return_value = getReflectedTemp(temperatureReflected);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Reflected temperature: " << temperatureReflected << std::endl;

	int year, month, day, hour, minute, second;
	return_value = getDateTime(year, month, day, hour, minute, second);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Date and time in WIRIS device: " << year << "/" << month << "/" << day << " " << hour << ":" << minute << ":" << second << std::endl;

	time_t t = time(0);
	struct tm * now = localtime( & t );
	year = now->tm_year;
	month = now->tm_mon;
	day = now->tm_mday;
	hour = now->tm_hour;
	minute = now->tm_min;
	second = now->tm_sec;
	return_value = setDateTime(year, month, day, hour, minute, second);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Setting actual date and time to WIRIS..." << std::endl;

	double spacePerCent;
	return_value = getFreeMemory(spacePerCent);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Free memory: " << spacePerCent << " %" << std::endl;

	int takenImages;
	return_value = getNumberImages(takenImages);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Taken images: " << takenImages << std::endl;

	int sequenceLength;
	return_value = getSequenceLength(sequenceLength);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Taken sequence: " << sequenceLength << " s" << std::endl;

	return_value = setCaptureRadiometric(true);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Set capture radiometric JPEG: ON" << std::endl;

	return_value = setCaptureTiff(true);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Set capture radiometric TIFF: ON" << std::endl;

	return_value = setCaptureVisible(true);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Set capture visible JPEG: ON" << std::endl;

	return_value = setCaptureScreen(true);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Set capture non-radiometric JPEG: ON" << std::endl;

	return_value = setPalette("blackred");
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Setting palette for thermal images: BlackRed" << std::endl;

	return_value = setRangeManual(true);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Setting thermal range to manual" << std::endl;

	return_value = setRangeMinimum(10.0);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Setting thermal range minimum to " << 10.0 << std::endl;

	return_value = setRangeMaximum(40.0);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "	- Setting thermal range maximum to " << 40.0 << std::endl;

	double latitude, longitude, altitude;
	bool status;
	return_value = getGPS(status, latitude, longitude, altitude);
	if(return_value != 0) {
		return -1;
	}
	if(status) {
		std::cout << "	- GPS position: " << latitude << "°, " << longitude << "°, " << altitude << "m" << std::endl;
	} else {
		std::cout << "	- No GPS signal" << std::endl;
	}


	/**************************************************************************************************/
	return_value = captureImages();
	if(return_value != 0) {
		return -1;
	}
	std::cout << "Saving WIRIS images..." << std::endl;
	sleep(1);

	return_value = startSequence();
	if(return_value != 0) {
		return -1;
	}
	std::cout << "Starting to capture sequence..." << std::endl;
	sleep(rand() % 3 + 1);

	bool capturing;
	return_value = getSequence(capturing);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "Capturing sequence: " << capturing << std::endl;

	return_value = stopSequence();
	if(return_value != 0) {
		return -1;
	}
	std::cout << "Finishing sequence..." << std::endl;
	usleep(250000);

	return_value = getSequence(capturing);
	if(return_value != 0) {
		return -1;
	}
	std::cout << "Capturing sequence: " << capturing << std::endl;

	/**************************************************************************************************/
	uint16_t buf_raw[widthThermo * heightThermo];
	uint8_t buf_img[widthThermo * heightThermo];
	uint8_t buf_vis[widthVisible * heightVisible * 2];

	std::cout << "Getting and saving thermo image." << std::endl;
	return_value = getFrameThermo(buf_raw, widthThermo, heightThermo);
	std::cout << "Temperature of first pixel: " << rawToTemperatureC(buf_raw[0]) << "°C" << std::endl;

	prepareGreyImage(buf_raw, buf_img, widthThermo, heightThermo);
	saveGrayJpeg("frame_thermo-1.jpg", buf_img, widthThermo, heightThermo);

	std::cout << "Getting and saving thermo image - numbered." << std::endl;
	return_value = getFrameThermoNumbered(buf_raw, widthThermo, heightThermo);
	std::cout << "Temperature of first pixel: " << rawToTemperatureC(buf_raw[0]) << "°C" << std::endl;

	prepareGreyImage(buf_raw, buf_img, widthThermo, heightThermo);
	saveGrayJpeg("frame_thermo-2.jpg", buf_img, widthThermo, heightThermo);

	std::cout << "Getting and saving visible image." << std::endl;
	return_value = getFrameVisible(buf_vis, widthVisible, heightVisible);
	saveVisibleJpeg("frame_visible-1.jpg", buf_vis, widthVisible, heightVisible);

	std::cout << "Getting and saving visible image - numbered." << std::endl;
	return_value = getFrameVisibleNumbered(buf_vis, widthVisible, heightVisible);
	saveVisibleJpeg("frame_visible-2.jpg", buf_vis, widthVisible, heightVisible);


	std::cout << "Testing stream speed:" << std::endl;
	time_t t_start = time(0);
	for (int i = 0; i < 50; ++i) {
		getFrameThermoNumbered(buf_raw, widthThermo, heightThermo);
	}
	int seconds = difftime(time(0), t_start);
	std::cout << "	- 50 thermo frames took " << seconds << " s" << std::endl;

	t_start = time(0);
	for (int i = 0; i < 50; ++i) {
		getFrameVisibleNumbered(buf_vis, widthVisible, heightVisible);
	}
	seconds = difftime(time(0), t_start);
	std::cout << "	- 50 visible frames took " << seconds << " s" << std::endl;

	/**************************************************************************************************/
	//shutDownWiris();
	//rebootWiris();
	std::cout << "Ending ethernet mode..." << std::endl;
	return_value = stopEthernetMode();
	if(return_value != 0) {
		return -1;
	}

	std::cout << "Closing." << std::endl;
	closeControl();
	closeThermo();
	closeVisible();
	std::cout << "Done." << std::endl;

	return 0;
}
